package de.mlessmann.certassist.service;

import de.mlessmann.certassist.Constants;
import de.mlessmann.certassist.DeleteRecursiveFileVisitor;
import de.mlessmann.certassist.openssl.CertificateProvider;
import de.mlessmann.certassist.openssl.CertificateUsage;
import de.mlessmann.certassist.repositories.CertificateRepository;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
@RequiredArgsConstructor
public class CertificateProviderImpl implements CertificateProvider {

    private static final OpenOption[] CREATE_TRUNCATE = new OpenOption[] {
        StandardOpenOption.CREATE,
        StandardOpenOption.TRUNCATE_EXISTING,
    };
    private final CertificateRepository certificateRepository;

    @Override
    public CertificateUsage requestCertificateUsage(String fingerprint) {
        var certFromDB = certificateRepository.findByFingerprintIs(fingerprint);
        if (certFromDB == null) {
            throw new IllegalArgumentException("Unknown fingerprint");
        }

        try {
            Path tempDirectory = Files.createTempDirectory(Constants.CERTASSIST_TMP_PREFIX);
            Files.write(tempDirectory.resolve("key.pem"), certFromDB.getPrivateKey(), CREATE_TRUNCATE);
            Files.write(tempDirectory.resolve("cert.pem"), certFromDB.getCert(), CREATE_TRUNCATE);
            return new ExtractedCert(tempDirectory, certFromDB.getFingerprint());
        } catch (IOException e) {
            // TODO: Better exception definitions
            throw new RuntimeException("Unable to temporarily store certificate for use.", e);
        }
    }

    @Slf4j
    private record ExtractedCert(Path tempDir, String fingerprint) implements CertificateUsage {
        @Override
        public Path certificateKeyPath() {
            return this.tempDir.resolve("key.pem");
        }

        @Override
        public Path certificatePath() {
            return this.tempDir.resolve("cert.pem");
        }

        @Override
        public void close() {
            try {
                Files.walkFileTree(this.tempDir, new DeleteRecursiveFileVisitor());
                Files.deleteIfExists(this.tempDir);
            } catch (IOException e) {
                log.error("Unable to clean up temporary directory: " + this.tempDir, e);
            }
        }
    }
}
